# 2013-08-07 09:43
# Demonstrate result for quantization error using MCM.

# Quantizing function for single values.
# d is the v-floor(v) "true" value.
# t is the relative time of the last sample.
# d and t must be in the range 0 to 1.
quant <- function(d,t) {
  if (t < d)
    1
  else
    0
}

# Vectorize everything to avoid confusing apply functions.
M = 100000
d = seq(0,1,0.05)
rd = rep(d,each=M)
y1 = runif(length(rd), 0.0, 1.0)
y2 = mapply(quant, rd, y1)

# Devectorize it for the results.
r = matrix(y2, ncol=M, byrow=TRUE)

# rowMeans(r) should approximate the original d's.
print("means")
rm = rowMeans(r)
print(rm)
print("variances")
temp = r-rm
temp = temp*temp
var = rowSums(temp) / (M-1)
print(var)

# model
pvar = d-d^2

png("quantsim.png",width=800,height=800,units="px",pointsize=16)
par(mar=c(4,4,0,0.1))
plot(x=d,y=pvar,pch=20,col="red",xlab=expression(paste("d = ",v-group(lfloor,v,rfloor)," or fmod(v,1.0)")),ylab="Variance")
points(x=d,y=var,pch=3,col="black")
legend("topleft",legend=c(expression(paste("Model ",(d-d^2))),"MCM result"),col=c("red","black"),pch=c(20,3))
dev.off()
