# 2013-09-03 15:22

# Requires plotrix package; tested plotrix_3.4-7.tar.gz
library(plotrix)

d <- read.table("covs.txt",header=T)
d <- d[order(d$f1),]
d$holespct <- d$holes / 10
xrange <- c(min(d$f1),max(d$f1))
yrange <- c(0,100)
legatt <- c("normatt","basicatt","percatt","bcaatt")
legu <- c("normu","basicu","percu","bcau")

makefile <- function(ispdf) {
  if (ispdf) {
    pdf("attained-coverage.pdf")
    par(mar=c(3,3,0.1,0.1),mgp=c(2,0.75,0))
    cexval <- 0.7
    arrlen <- 0.04
  } else {
    png("attained-coverage.png",width=800,height=800,units="px",pointsize=16)
    par(mar=c(3,3,0.1,0.1),mgp=c(2,0.75,0))
    cexval <- 1.0
    arrlen <- 0.05
  }
  plot(xrange,yrange,type="n",xlab="Frequency of secondary mode",ylab="% attained coverage and % degenerate cases",xaxt="n")
  axis(1,at=seq(0.001,0.01,0.001))
  colors <- rainbow(5,v=0.85)
  for (i in 1:4) {
    lines(d$f1,d[,legatt[i]],type="b",lwd=1.5,lty=1,col=colors[i],pch=16,cex=cexval)
    arrows(d$f1,d[,legatt[i]]-d[,legu[i]],d$f1,d[,legatt[i]]+d[,legu[i]],angle=90,code=3,length=arrlen,col=colors[i])
  }
  lines(d$f1,d$holes/10,type="b",lwd=1.5,lty=1,col=colors[5],pch=16,cex=cexval)
  abline(h=95,col="black",lty=2)
  legend(0.005,60,legend=c("Normal coverage interval","Basic coverage interval","Percentile coverage interval","BCa coverage interval","Proportion of degenerate cases"),col=colors,pch=16,lty=1)
  # This didn't work
  # paste(expression(BC[a])," coverage interval",sep="")
  dev.off()
}

makefile(F)
makefile(T)
