# 2013-09-12 13:56
# bootstrap-run.R runs MCM and produces bootstrap-data-summary.txt.
# This script reads that file and produces the plots.

# Requires plotrix package; tested plotrix_3.4-7.tar.gz
library(plotrix)

stats <- read.table("bootstrap-data-summary.txt",header=T)

# Messy retrofit.
# For Multinomial predictions we use pooled values for the p's and the mean
# of the big-n sample counts.
bign <- read.table("data-bign.txt",header=T)
lstats <- aggregate(bign, list(logq2=bign$logq2,hz=bign$hz), FUN=sum)[,c(1,2,5)]
temp <- aggregate(bign, list(logq2=bign$logq2,hz=bign$hz), FUN=length)[,c(1,2,5)]
colnames(temp)[3] <- "count";
lstats <- merge(lstats, temp)
data <- read.table("data-denormalized.txt",header=T)
temp <- aggregate(data, list(logq2=data$logq2,hz=data$hz,p=data$p), FUN=sum)[,c(1,2,3,7)]
colnames(temp)[4] <- "p0";
lstats <- merge(lstats, temp[temp$p==0,])[,c(1,2,3,4,6)]
colnames(temp)[4] <- "p1";
lstats <- merge(lstats, temp[temp$p==1,])[,c(1,2,3,4,5,7)]
lstats$meanbign <- lstats$bign / lstats$count
lstats$p0pooled <- lstats$p0 / lstats$bign
lstats$p1pooled <- lstats$p1 / lstats$bign
lstats$p0lpredsd <- sqrt(lstats$meanbign * lstats$p0pooled * (1-lstats$p0pooled))
lstats$p1lpredsd <- sqrt(lstats$meanbign * lstats$p1pooled * (1-lstats$p1pooled))
temp1 <- lstats[,c("logq2","hz","p0lpredsd")]
colnames(temp1)[3] <- "lpredsd"
temp1$p = 0
temp2 <- lstats[,c("logq2","hz","p1lpredsd")]
colnames(temp2)[3] <- "lpredsd"
temp2$p = 1
temp <- rbind(temp1,temp2)
stats <- merge(stats,temp)

# logq2 to log10(calls) transformation
callcount <- function(logq2){
  return (1.0+logq2/2.0)
}

safetymargin <- 2   # Not needed on outside margins
cutoff <- (max(stats$mean)+min(stats$mean))/2.0
gapy1 <- max(stats$mean[stats$mean < cutoff])+safetymargin
gapy2 <- min(stats$mean[stats$mean > cutoff])-safetymargin
gaplen <- gapy2-gapy1
ylim1 <- min(stats$mean)
ylim2 <- max(stats$mean)-gaplen
xlim1 <- callcount(min(stats$logq2))
xlim2 <- callcount(max(stats$logq2))
xlims <- c(xlim1,xlim2)
ylims <- c(ylim1,ylim2)

# y coordinate gap transformation
ygap <- function(y,gaplen,cutoff) {
  return (ifelse(y<cutoff,y,y-gaplen))
}

makemeanfile <- function(ispdf) {
  if (ispdf) {
    pdf("mean-bootstrap.pdf")
    par(mar=c(4,4,3.5,1))
    cexval <- 0.8
    arrlen <- 0.06
    legend_vertical_fudge              <- 0.12  # lines of text? (+ = up)
    right_side_labels_horizontal_fudge <- 0.38  # x-axis units (+ = right)
  } else {
    png("mean-bootstrap.png",width=800,height=800,units="px",pointsize=16)
    par(mar=c(4,4,5,1))
    cexval <- 1.0
    arrlen <- 0.1
    legend_vertical_fudge              <- 0.105  # lines of text? (+ = up)
    right_side_labels_horizontal_fudge <- 0.375  # x-axis units (+ = right)
  }

  # gap, xlim, and ylim must be repeated for each one.
  gap.plot (callcount(stats$logq2[stats$hz == 1]),stats$mean[stats$hz == 1],gap=c(gapy1,gapy2),xlim=xlims,ylim=ylims,pch=16,cex=cexval,col="black",xlab="log10 of number of function calls per program run",ylab="Mean sample count",ytics=c(30,35,40,45,115,120,125,130,135))
  gap.plot (callcount(stats$logq2[stats$hz == 0]),stats$mean[stats$hz == 0],gap=c(gapy1,gapy2),xlim=xlims,ylim=ylims,pch=16,cex=cexval,col="red",add=TRUE)
  title ("Gprof mean sample counts",line=4)

  # Coverage interval of the means
  arrows(callcount(stats$logq2[stats$hz==1]),ygap(stats$meanlo[stats$hz==1],gaplen,cutoff),callcount(stats$logq2[stats$hz==1]),ygap(stats$meanhi[stats$hz==1],gaplen,cutoff),angle=90,code=3,length=arrlen,col="black")
  arrows(callcount(stats$logq2[stats$hz==0]),ygap(stats$meanlo[stats$hz==0],gaplen,cutoff),callcount(stats$logq2[stats$hz==0]),ygap(stats$meanhi[stats$hz==0],gaplen,cutoff),angle=90,code=3,length=arrlen,col="red")

  text(xpd=TRUE,x=xlim2+right_side_labels_horizontal_fudge,y=(gapy2+ylim2-gaplen)/2,srt=270,labels=c("fn2, 8/10 calls"))
  text(xpd=TRUE,x=xlim2+right_side_labels_horizontal_fudge,y=(gapy1+ylim1)/2,srt=270,labels=c("fn1, 2/10 calls"))

  legend("top",legend=c("HZ=100","HZ=1000"),pch=c(16,16),col=c("red","black"),inset=c(0,-legend_vertical_fudge),xpd=TRUE)
  dev.off()
}

makesdfile <- function(ispdf) {
  if (ispdf) {
    pdf("sd-bootstrap.pdf")
    par(mar=c(4,4,0,0.1))
    cexval <- 0.8
    arrlen <- 0.06
    plot(callcount(stats$logq2[stats$hz==0 & stats$p==0]),stats$predsd[stats$hz==0 & stats$p==0],xlab="log10 of number of function calls per program run",ylab="Standard deviation of sample counts",pch=1,cex=cexval,col="red",ylim=c(0,max(stats$predsd,stats$sdhi)))
  } else {
    png("sd-bootstrap.png",width=800,height=800,units="px",pointsize=16)
    par(mar=c(4,4,2,0))
    cexval <- 1.0
    arrlen <- 0.1
    plot(callcount(stats$logq2[stats$hz==0 & stats$p==0]),stats$predsd[stats$hz==0 & stats$p==0],xlab="log10 of number of function calls per program run",ylab="Standard deviation of sample counts",pch=1,cex=cexval,col="red",main="Sample count standard deviations, predicted vs. actual",ylim=c(0,max(stats$predsd,stats$sdhi)))
  }

  # Gprof predicted sds (plus the one included in the plot command above)
  points(callcount(stats$logq2[stats$hz==1 & stats$p==0]),stats$predsd[stats$hz==1 & stats$p==0],pch=1,cex=cexval,col="black")
  points(callcount(stats$logq2[stats$hz==0 & stats$p==1]),stats$predsd[stats$hz==0 & stats$p==1],pch=0,cex=cexval,col="red")
  points(callcount(stats$logq2[stats$hz==1 & stats$p==1]),stats$predsd[stats$hz==1 & stats$p==1],pch=0,cex=cexval,col="black")

  # Multinomial predicted sds
  points(callcount(stats$logq2[stats$hz==0 & stats$p==0]),stats$lpredsd[stats$hz==0 & stats$p==0],pch=1,cex=cexval*0.75,col="red")
  points(callcount(stats$logq2[stats$hz==1 & stats$p==0]),stats$lpredsd[stats$hz==1 & stats$p==0],pch=1,cex=cexval*0.75,col="black")
  points(callcount(stats$logq2[stats$hz==0 & stats$p==1]),stats$lpredsd[stats$hz==0 & stats$p==1],pch=0,cex=cexval*0.75,col="red")
  points(callcount(stats$logq2[stats$hz==1 & stats$p==1]),stats$lpredsd[stats$hz==1 & stats$p==1],pch=0,cex=cexval*0.75,col="black")

  # Actual sds
  points(callcount(stats$logq2[stats$hz==0 & stats$p==0]),stats$sd[stats$hz==0 & stats$p==0],pch=16,cex=cexval,col="red")
  points(callcount(stats$logq2[stats$hz==1 & stats$p==0]),stats$sd[stats$hz==1 & stats$p==0],pch=16,cex=cexval,col="black")
  points(callcount(stats$logq2[stats$hz==0 & stats$p==1]),stats$sd[stats$hz==0 & stats$p==1],pch=15,cex=cexval,col="red")
  points(callcount(stats$logq2[stats$hz==1 & stats$p==1]),stats$sd[stats$hz==1 & stats$p==1],pch=15,cex=cexval,col="black")

  # Coverage intervals of the sds
  arrows(callcount(stats$logq2[stats$hz==0]),stats$sdlo[stats$hz==0],callcount(stats$logq2[stats$hz==0]),stats$sdhi[stats$hz==0],angle=90,code=3,length=arrlen,col="red")
  arrows(callcount(stats$logq2[stats$hz==1]),stats$sdlo[stats$hz==1],callcount(stats$logq2[stats$hz==1]),stats$sdhi[stats$hz==1],angle=90,code=3,length=arrlen,col="black")

  if (ispdf) {
    legend(1.25,10,legend=c("red:            HZ=100", 
                            "black:         HZ=1000",
                            "circles:       fn1 with 2/10 calls",
                            "squares:    fn2 with 8/10 calls",
                            "filled:              observed",
 			    "open, big:       predicted (gprof)",
			    "open, small:   predicted (multinomial)"))
  } else {
    legend(1.5,10,legend=c("red:             HZ=100", 
	  		   "black:         HZ=1000",
			   "circles:       fn1 with 2/10 calls",
			   "squares:    fn2 with 8/10 calls",
			   "filled:               observed",
			   "open, big:       predicted (gprof)",
			   "open, small:   predicted (multinomial)"))
  }
  dev.off()
}

makesdfile(F)
makesdfile(T)
makemeanfile(F)
makemeanfile(T)
