#!/usr/bin/ruby
# 2013-09-11 11:21

# This does not do the corrected sample standard deviation (n-1).
# descriptive_statistics-1.1.1.gem from https://github.com/thirtysixthspan/descriptive_statistics 2013-05-14 13:58
# gem install descriptive_statistics-1.1.1.gem
# require 'descriptive_statistics'

def sum(x)
  x.inject(:+).to_f
end

def mean(x)
  sum(x)/x.size
end

def sd(x)
  xm = mean(x)
  v = x.map{ |sample| (xm - sample) ** 2 }.inject(:+) / (x.size - 1.0)
  Math.sqrt(v)
end

# convert returns a triplet (actually an Array of size 3) of Arrays.  The
# first one contains the sample counts from fn1; the second from fn2; the
# third is the total sample count of the run.
def convert(logq2,hz)
  triplet = Array.new(3) { Array.new }
  q = (10 ** (logq2/2.0) + 0.5).to_i
  fname = "q=" + q.to_s + "_HZ=" + hz + "_gprof.txt"
  puts "Reading " + fname
  File.open(fname,"r") do |file|
    state = 0
    bign = -1
    file.lines do |line|
      if state == 0
	raise "Gprof input format error 0" if line.chomp! != "Flat profile:"
	state = 1
      elsif state == 1
	state = 2
      elsif state == 2
	raise "Gprof input format error 1" if line.chomp! != "Each sample counts as 0.01 seconds."
	state = 3
      elsif state < 5
	state += 1
      else
	if "1 ".include? line[0]
	  ls = line.split(' ')
          # bign is taken from the cumulative seconds field of the last
          # function listed in each profile.
          bign = (ls[1].to_r * 100).to_i
          samplecount = (ls[2].to_r * 100).to_i
	  case ls.last
	    when "fn1" then triplet[0].push(samplecount)
	    when "fn2" then triplet[1].push(samplecount)
          end
	else
	  raise "Gprof input format error 2" if line.chomp! != "Flat profile:"
          triplet[2].push(bign)
	  state = 1
	end
      end
    end
    triplet[2].push(bign) if state == 5
  end
  # FIXME for p=0
  raise "Sample count mismatch 1" if triplet[0].size != triplet[1].size
  raise "Sample count mismatch 2" if triplet[0].size != triplet[2].size
  raise "Wrong sample count" if triplet[0].size != 1000
  triplet
end

# i & 0F  = logq*2  (0..14, 15 is unused)
# i & 10  = hz  (0=100,1=1000)
# i & 20  = p   (0=0.2(fn1), 1=0.8(fn2))
data = Array.new(63) { Array.new }
bign = Array.new(31) { Array.new }
(0..31).each do |i|
  logq2 = i&0xF
  if logq2 < 15
    hz = i&0x10==0 ? "100" : "1000"
    triplet = convert(logq2,hz)
    data[i] = triplet[0]
    data[i+0x20] = triplet[1]
    bign[i] = triplet[2]
  end
end

crunched = File.open("data-summary.txt","w")
crunched.puts "logq2 hz p count mean sd"
dump = File.open("data-denormalized.txt","w")
dump.puts "logq2 hz p sample"
bignfile = File.open("data-bign.txt","w")
bignfile.puts "logq2 hz bign"
(0..62).each do |i|
  logq2 = i&0xF
  if logq2 < 15
    hz = i&0x10==0 ? "0" : "1"
    p  = i&0x20==0 ? "0" : "1"
    crunched.puts logq2.to_s + " " + hz + " " + p + " " + data[i].size.to_s + " " + mean(data[i]).to_s + " " + sd(data[i]).to_s
    if (i < 31)
      bign[i].each {|x| bignfile.print logq2, " ", hz, " ", x, "\n" }
    end
    data[i].each {|x| dump.print logq2, " ", hz, " ", p, " ", x, "\n" }
  end
end
