#!/usr/bin/ruby
# 2013-07-08 11:49

# Number of samples for each data set
$N = 959

# If true, stop at $N or end of file, whichever comes first.
# If false, read to end of file.
# Either way, complain if don't get $N samples for everything.
$TRUNCATE = true

# Number of functions that are being tracked.
$NUMGLOM = 2

# This does not do the corrected sample standard deviation (n-1).
# descriptive_statistics-1.1.1.gem from https://github.com/thirtysixthspan/descriptive_statistics 2013-05-14 13:58
# gem install descriptive_statistics-1.1.1.gem
# require 'descriptive_statistics'

def sum(x)
  x.inject(:+).to_f
end

def mean(x)
  sum(x)/x.size
end

def sd(x)
  xm = mean(x)
  v = x.map{ |sample| (xm - sample) ** 2 }.inject(:+) / (x.size - 1.0)
  Math.sqrt(v)
end

def fncount2bucketi(fncount)
  fncount / $NUMGLOM
end

# convert returns a pair (actually an Array of size 2) of Arrays.  The first
# one contains the sample counts from fn1; the second from fn2.
def convert(logq2)
  pair = Array.new(2) { Array.new }
  q = (10 ** (logq2/2.0) + 0.5).to_i
  fname = "q=" + q.to_s + "_AXE.txt"
  puts "Reading " + fname
  File.open(fname,"r") do |file|
    fncount = 0
    file.lines do |line|
      line.chomp!
      if line[0..1] == "fn"
        i = line[2].to_i - 1
        raise "AXE format error" if i < 0 or i > 1
        ms = (line.split(' ')[2].to_f * 1000 + 0.5).to_i
        # Approximate conversion to sample count
        samples = 1.4 * ms
        pair[i].push(samples)
        fncount += 1
        break if $TRUNCATE and fncount2bucketi(fncount) == $N
      end
    end
  end
  # FIXME for p=0
  raise "Sample count mismatch" if pair[0].size != pair[1].size
  raise "Wrong sample count" if pair[0].size != $N
  pair
end

# i & 0F  = logq*2  (0..14, 15 is unused)
# i & 10  = p   (0=0.2(fn1), 1=0.8(fn2))
data = Array.new(31) { Array.new }
(0..14).each do |i|
  pair = convert(i)
  data[i] = pair[0]
  data[i+0x10] = pair[1]
end

crunched = File.open("data-summary.txt","w")
crunched.puts "logq2 p count mean sd"
(0..30).each do |i|
  logq2 = i&0xF
  if logq2 < 15
    p  = i&0x10==0 ? "0" : "1"
    crunched.puts logq2.to_s + " " + p + " " + data[i].size.to_s + " " + mean(data[i]).to_s + " " + sd(data[i]).to_s
  end
end
