# 2013-08-15 10:57

options(warn=1)

# logq2 to log10(calls) transformation
callcount <- function(logq2){
  return (1.0+logq2/2.0)
}

data <- read.table("data-denormalized.txt",header=T)
IQRs <- aggregate(data, list(logq2=data$logq2,p=data$p), FUN=IQR)[,c(1,2,5)]
SDs <- aggregate(data, list(logq2=data$logq2,p=data$p), FUN=sd)[,c(1,2,5)]
IQRs$sample = IQRs$sample / 1.349

ylim1 <- min(IQRs$sample,SDs$sample)
ylim2 <- max(IQRs$sample,SDs$sample)
xlim1 <- callcount(min(IQRs$logq2))
xlim2 <- callcount(max(IQRs$logq2))
xlims <- c(xlim1,xlim2)
ylims <- c(ylim1,ylim2)

makefile <- function(ispdf) {
  if (ispdf) {
    pdf("combine.pdf")
    par(mar=c(4,4,0,0.1))
    cexval <- 0.8
  } else {
    png("combine.png",width=800,height=800,units="px",pointsize=16)
    par(mar=c(4,4,2,0))
    cexval <- 1.0
  }

  plot (callcount(SDs$logq2[SDs$p==0]),SDs$sample[SDs$p==0],xlim=xlims,ylim=ylims,pch=16,cex=cexval,col="black",xlab="log10 of number of function calls per program run",ylab="SD and IQR/1.349 of approximate sample counts")
  points (callcount(IQRs$logq2[IQRs$p==0]),IQRs$sample[IQRs$p==0],pch=16,cex=cexval,col="red")

  points (callcount(SDs$logq2[SDs$p==1]),SDs$sample[SDs$p==1],pch=16,cex=cexval,col="blue")
  points (callcount(IQRs$logq2[IQRs$p==1]),IQRs$sample[IQRs$p==1],pch=16,cex=cexval,col="darkgoldenrod4")

  title ("AXE approximate sample counts SDs and IQRs",line=1)
  legend(1,19,legend=c("SD, fn1 with 2/10 calls",
                       "IQR/1.349, fn1 with 2/10 calls",
                       "SD, fn2 with 8/10 calls",
                       "IQR/1.349, fn2 with 8/10 calls"),pch=c(16,16,16,16),
         col=c("black","red","blue","darkgoldenrod4"))
  dev.off()
}

makefile(F)
makefile(T)
