#!/usr/bin/ruby
# 2013-09-11 15:10

# Number of samples for each data set
$N = 1019

# If true, stop at $N or end of file, whichever comes first.
# If false, read to end of file.
# Either way, complain if don't get $N samples for everything.
$TRUNCATE = true

# Number of functions that are being tracked.
$NUMGLOM = 2

# This does not do the corrected sample standard deviation (n-1).
# descriptive_statistics-1.1.1.gem from https://github.com/thirtysixthspan/descriptive_statistics 2013-05-14 13:58
# gem install descriptive_statistics-1.1.1.gem
# require 'descriptive_statistics'

def sum(x)
  x.inject(:+).to_f
end

def mean(x)
  sum(x)/x.size
end

def sd(x)
  xm = mean(x)
  v = x.map{ |sample| (xm - sample) ** 2 }.inject(:+) / (x.size - 1.0)
  Math.sqrt(v)
end

def fncount2bucketi(fncount)
  fncount / $NUMGLOM
end

# convert returns a triplet (actually an Array of size 3) of Arrays.  The
# first one contains the sample counts from fn1; the second from fn2; the
# third is the total sample count of the run.
def convert(logq2)
  triplet = Array.new(3) { Array.new }
  q = (10 ** (logq2/2.0) + 0.5).to_i
  fname = "q=" + q.to_s + "_AXE.txt"
  puts "Reading " + fname
  File.open(fname,"r") do |file|
    fncount = 0
    state = 0
    bign = 0
    file.lines do |line|
      line.chomp!
      if line.length > 0 and line[0..7] != "Function" and line[0] != "-"
        state = 1
        ms = (line.split(' ')[2].to_f * 1000 + 0.5).to_i
        # Approximate conversion to sample count
        samples = 1.3 * ms
        bign += samples
        if line[0..1] == "fn"
          i = line[2].to_i - 1
          raise "AXE format error" if i < 0 or i > 1
          triplet[i].push(samples)
          fncount += 1
          break if $TRUNCATE and fncount2bucketi(fncount) == $N
        end
      else
        if state == 1
          triplet[2].push(bign)
          bign = 0
          state = 0
        end
      end
    end
    if state == 1
      triplet[2].push(bign)
    end
  end
  # FIXME for p=0
  raise "Sample count mismatch 1" if triplet[0].size != triplet[1].size
  raise "Sample count mismatch 2" if triplet[0].size != triplet[2].size
  raise "Wrong sample count" if triplet[0].size != $N
  triplet
end

# i & 0F  = logq*2  (0..14, 15 is unused)
# i & 10  = p   (0=0.2(fn1), 1=0.8(fn2))
data = Array.new(31) { Array.new }
bign = Array.new(15) { Array.new }
(0..14).each do |i|
  triplet = convert(i)
  data[i] = triplet[0]
  data[i+0x10] = triplet[1]
  bign[i] = triplet[2]
end

crunched = File.open("data-summary.txt","w")
crunched.puts "logq2 p count mean sd"
dump = File.open("data-denormalized.txt","w")
dump.puts "logq2 p sample"
bignfile = File.open("data-bign.txt","w")
bignfile.puts "logq2 bign"
(0..30).each do |i|
  logq2 = i&0xF
  if logq2 < 15
    p  = i&0x10==0 ? "0" : "1"
    crunched.puts logq2.to_s + " " + p + " " + data[i].size.to_s + " " + mean(data[i]).to_s + " " + sd(data[i]).to_s
    if (i < 15)
      bign[i].each {|x| bignfile.print logq2, " ", x, "\n" }
    end
    data[i].each {|x| dump.print logq2, " ", p, " ", x, "\n" }
  end
end
