#!/usr/bin/ruby
# 2013-07-03 11:15

# Number of samples for each data set
$N = 1019

# If true, stop at $N or end of file, whichever comes first.
# If false, read to end of file.
# Either way, complain if don't get $N samples for everything.
# (If count > $N and $TRUNCATE == false, you get "Bad bucket.")
$TRUNCATE = true

# Number of functions that are being combined.
$NUMGLOM = 2

$sums   = Array.new($N, 0.0)
$counts = Array.new($N, 0)

def fncount2bucketi(fncount)
  fncount / $NUMGLOM
end

def convert(logq2)
  q = (10 ** (logq2/2.0) + 0.5).to_i
  fname = "q=" + q.to_s + "_AXE.txt"
  puts "Reading " + fname
  File.open(fname,"r") do |file|
    fncount = 0
    file.lines do |line|
      line.chomp!
      if line[0..1] == "fn"
        i = line[2].to_i - 1
        raise "AXE format error" if i < 0 or i > 1
        ms = (line.split(' ')[2].to_f * 1000 + 0.5).to_i
        # Approximate conversion to sample count
        # samples = 1.3 * ms
        # We don't even care which function it was, just which bucket.
        # Have faith that we got fn1 and fn2 once for each run.
        bucketi = fncount2bucketi(fncount)
        raise "Bad bucket" if bucketi >= $N
        $sums[bucketi] += ms
        $counts[bucketi] += 1
        fncount += 1
        return if $TRUNCATE and fncount2bucketi(fncount) == $N
      end
    end
  end
end

(0..14).each do |i|
  convert(i)
end

# 15 data files times $NUMGLOM functions per run that were glommed together
raise "Wrong count" if $counts.detect { |x| x != 15 * $NUMGLOM }

crunched = File.open("glommed-noscale-time-series.txt","w")
crunched.puts "ms"
crunched.puts $sums
