# 2013-08-21 13:37

# Requires plotrix package; tested plotrix_3.4-7.tar.gz
library(plotrix)

# logq2 to log10(calls) transformation
callcount <- function(logq2){
  return (1.0+logq2/2.0)
}

data <- read.table("data-summary.txt",header=T)
safetymargin <- 20   # Not needed on outside margins
cutoff <- (max(data$mean)+min(data$mean))/2.0
gapy1 <- max(data$mean[data$mean < cutoff])+safetymargin
gapy2 <- min(data$mean[data$mean > cutoff])-safetymargin
gaplen <- gapy2-gapy1
ylim1 <- min(data$mean)
ylim2 <- max(data$mean)-gaplen
xlim1 <- callcount(min(data$logq2))
xlim2 <- callcount(max(data$logq2))
xlims <- c(xlim1,xlim2)
ylims <- c(ylim1,ylim2)

# 95% coverage interval for the means
alpha <- 0.05
dof <- data$count[1] - 1
semdiv <- sqrt(data$count[1])
k <- qt(1-alpha/2,dof)
data$meanlo <- data$mean - k * data$sd / semdiv
data$meanhi <- data$mean + k * data$sd / semdiv

# y coordinate gap transformation
ygap <- function(y,gaplen,cutoff) {
  return (ifelse(y<cutoff,y,y-gaplen))
}

makefile <- function(ispdf) {
  if (ispdf) {
    pdf("mean-orig.pdf")
    par(mar=c(4,4,0,1))
    cexval <- 0.8
    arrlen <- 0.06
    right_side_labels_horizontal_fudge <- 0.38   # x-axis units (+ = right)
  } else {
    png("mean-orig.png",width=800,height=800,units="px",pointsize=16)
    par(mar=c(4,4,2,1))
    cexval <- 1.0
    arrlen <- 0.1
    right_side_labels_horizontal_fudge <- 0.375  # x-axis units (+ = right)
  }

  # gap, xlim, and ylim must be repeated for each one.
  gap.plot (callcount(data$logq2),data$mean,gap=c(gapy1,gapy2),xlim=xlims,ylim=ylims,pch=16,cex=cexval,col="black",xlab="log10 of number of function calls per program run",ylab="Mean approximate sample count",ytics=c(600,650,2400,2450,2500,2550,2600))
  title ("AXE mean sample counts",line=1)

  # Coverage intervals of the means
  arrows(callcount(data$logq2),ygap(data$meanlo,gaplen,cutoff),callcount(data$logq2),ygap(data$meanhi,gaplen,cutoff),angle=90,code=3,length=arrlen,col="black")

  text(xpd=TRUE,x=xlim2+right_side_labels_horizontal_fudge,y=(gapy2+ylim2-gaplen)/2,srt=270,labels=c("fn2, 8/10 calls"))
  text(xpd=TRUE,x=xlim2+right_side_labels_horizontal_fudge,y=(gapy1+ylim1)/2,srt=270,labels=c("fn1, 2/10 calls"))
  dev.off()
}

makefile(F)
makefile(T)
