# 2013-08-19 15:03
# bootstrap-run.R runs MCM and produces bootstrap-data-summary.txt.
# This script reads that file and produces the plots.

options(warn=1)

# Requires plotrix package; tested plotrix_3.4-7.tar.gz
library(plotrix)

stats <- read.table("bootstrap-data-summary.txt",header=T)

# logq2 to log10(calls) transformation
callcount <- function(logq2){
  return (1.0+logq2/2.0)
}

ylim1 <- min(stats$mean)
ylim2 <- max(stats$mean)
xlim1 <- callcount(min(stats$logq2))
xlim2 <- callcount(max(stats$logq2))
xlims <- c(xlim1,xlim2)
ylims <- c(ylim1,ylim2)

makemeanfile <- function(ispdf) {
  if (ispdf) {
    pdf("mean-bootstrap.pdf")
    par(mar=c(4,4,0,0.1))
    cexval <- 0.8
    arrlen <- 0.06
    right_side_labels_horizontal_fudge <- 0.38   # x-axis units (+ = right)
  } else {
    png("mean-bootstrap.png",width=800,height=800,units="px",pointsize=16)
    par(mar=c(4,4,2,0.1))
    cexval <- 1.0
    arrlen <- 0.1
    right_side_labels_horizontal_fudge <- 0.375  # x-axis units (+ = right)
  }

  plot (callcount(stats$logq2[stats$p==0]),stats$mean[stats$p==0],xlim=xlims,ylim=ylims,pch=16,cex=cexval,col="black",xlab="log10 of number of function calls per program run",ylab="Mean reported self time (ms)")
  points (callcount(stats$logq2[stats$p==1]),stats$mean[stats$p==1],pch=15,cex=cexval,col="black")
  title ("DroidX Dalvik Debug mean reported self times",line=1)

  legend(1,6e+3,legend=c("fn1 with 2/10 calls",
                         "fn2 with 8/10 calls"),pch=c(16,15))

  # Coverage intervals of the means
  arrows(callcount(stats$logq2),stats$meanlo,callcount(stats$logq2),stats$meanhi,angle=90,code=3,length=arrlen,col="black")
  dev.off()
}

makesdfile <- function(ispdf) {
  if (ispdf) {
    pdf("sd-bootstrap.pdf")
    par(mar=c(4,4,0,0.1))
    cexval <- 0.8
    arrlen <- 0.06
    legendcex <- 0.75
    plot(callcount(stats$logq2[stats$p==0]),stats$sd[stats$p==0],xlab="log10 of number of function calls per program run",ylab="Standard deviation of self times (ms)",pch=16,cex=cexval,col="black",xlim=xlims,ylim=c(0,max(stats$sd,stats$sdhi)))
  } else {
    png("sd-bootstrap.png",width=800,height=800,units="px",pointsize=16)
    par(mar=c(4,4,2,0))
    cexval <- 1.0
    arrlen <- 0.1
    legendcex <- 1.0
    plot(callcount(stats$logq2[stats$p==0]),stats$sd[stats$p==0],xlab="log10 of number of function calls per program run",ylab="Standard deviation of self times (ms)",pch=16,cex=cexval,col="black",main="Self time standard deviations",xlim=xlims,ylim=c(0,max(stats$sd,stats$sdhi)))
  }

  # Actual sds
  points(callcount(stats$logq2[stats$p==1]),stats$sd[stats$p==1],pch=15,cex=cexval,col="black")

  # Coverage intervals of the sds
  arrows(callcount(stats$logq2[stats$p==0]),stats$sdlo[stats$p==0],callcount(stats$logq2[stats$p==0]),stats$sdhi[stats$p==0],angle=90,code=3,length=arrlen,col="black")
  arrows(callcount(stats$logq2[stats$p==1]),stats$sdlo[stats$p==1],callcount(stats$logq2[stats$p==1]),stats$sdhi[stats$p==1],angle=90,code=3,length=arrlen,col="black")

  legend(1,20,legend=c("fn1 with 2/10 calls",
                       "fn2 with 8/10 calls"),pch=c(16,15))
  dev.off()
}

makesdfile(F)
makesdfile(T)
makemeanfile(F)
makemeanfile(T)
