# 2013-08-21 13:52
# This script runs MCM and produces bootstrap-data-summary.txt.
# bootstrap-plot.R reads that file and produces the plots.

# Configurable:  Number of Monte Carlo trials.
M <- 1000000

# Configurable:  Alpha (1 - coverage probability).
alpha <- 0.05

data <- read.table("data-denormalized.txt",header=T)
stats <- aggregate(data, list(logq2=data$logq2,p=data$p), FUN=mean)[,c(1,2,5)]
colnames(stats)[3] <- "mean";
# stats$predsd <- sqrt(stats$mean)
temp <- aggregate(data, list(logq2=data$logq2,p=data$p), FUN=sd)[,c(1,2,5)]
stats$sd <- temp$sample

outputplot <- function(parm,filename,data,interval) {
  # bw.nrd0 is the default estimator of density().
  bw <- bw.nrd0(data)
  png(filename,width=800,height=800,units="px",pointsize=16)
  par(mar=c(4,4,1,0.5))
  plot(density(data,bw=bw),lwd=2.5,xlab=parm,main=sprintf("Bandwidth = %0.4f", bw))
  abline(v=interval,col="blue")
  dev.off()
}

# Return:  estmean meanlo meanhi estsd sdlo sdhi
dogroup <- function(logq2,p) {
  gdata <- data[data$logq2==logq2 & data$p==p, 3]
  N <- length(gdata)

  cat(sprintf("Running %d Monte Carlo trials for logq2=%d p=%d (N=%d)...\n", M, logq2, p, N))
  means <- rep(NA, M)
  sds   <- rep(NA, M)
  for (i in 1:M) {
    sampvec  <- sample(gdata,N,replace=T)
    means[i] <- mean(sampvec)
    sds[i]   <- sd(sampvec)
  }
  quantiles <- c(alpha/2, 1-alpha/2)
  meanqs    <- quantile(means, quantiles)
  sdqs      <- quantile(sds, quantiles)

  # Save pictures of the output distributions with the coverage intervals.
  outputplot("sample count mean",sprintf("outputdist-mean-q%02d-p%d.png", logq2, p),means,meanqs)
  outputplot("sample count SD",sprintf("outputdist-SD-q%02d-p%d.png", logq2, p),sds,sdqs)

  c(mean(means),meanqs,mean(sds),sdqs)
}

for (logq2 in min(data$logq2):max(data$logq2)) {
  for (p in min(data$p):max(data$p)) {
    MCMout <- dogroup(logq2,p)
    stats$estmean[stats$logq2==logq2 & stats$p==p] <- MCMout[1]
    stats$meanlo[stats$logq2==logq2 & stats$p==p] <- MCMout[2]
    stats$meanhi[stats$logq2==logq2 & stats$p==p] <- MCMout[3]
    stats$estsd[stats$logq2==logq2 & stats$p==p] <- MCMout[4]
    stats$sdlo[stats$logq2==logq2 & stats$p==p] <- MCMout[5]
    stats$sdhi[stats$logq2==logq2 & stats$p==p] <- MCMout[6]
  }
}
statsout <- file("bootstrap-data-summary.txt",open="wt")
write.table(stats,file=statsout,quote=F,row.names=F)
close(statsout)
