#!/usr/bin/ruby
# 2013-07-22 13:38

# Number of samples for each data set.
# Range of completed runs is 1 to 569, but ak575_k-380.trace was unreadable.
# Skip iteration 380 for all levels.
$N = 568

# This does not do the corrected sample standard deviation (n-1).
# descriptive_statistics-1.1.1.gem from https://github.com/thirtysixthspan/descriptive_statistics 2013-05-14 13:58
# gem install descriptive_statistics-1.1.1.gem
# require 'descriptive_statistics'

def sum(x)
  x.inject(:+).to_f
end

def mean(x)
  sum(x)/x.size
end

def sd(x)
  xm = mean(x)
  v = x.map{ |sample| (xm - sample) ** 2 }.inject(:+) / (x.size - 1.0)
  Math.sqrt(v)
end

# Get a single pair of fn1 and fn2 counts from a single report file.
def getone(fname)
  pair = Array.new(2)
  puts "Reading " + fname
  File.open(fname,"r") do |file|
    file.lines do |line|
      ls = line.split(' ')
      usecs = ls[0].to_i
      if usecs > 0
        fn = ls[4]
        if fn[0..38] == "gov/nist/proportionate/proportionate.fn"
          i = fn[39].to_i - 1
          raise "dmtracedump format error" if i < 0 or i > 1
          raise "already saw this fn" unless pair[i].nil?
          pair[i] = usecs / 1000.0
          return pair unless pair[1-i].nil?
        end
      end
    end
  end
  raise "data not found"
end

# convert returns a pair (actually an Array of size 2) of Arrays.  The first
# one contains the sample counts from fn1; the second from fn2.
# Files are a..k (11 levels, not original 15) and 1..$N
def convert(logq2)
  pair = Array.new(2) { Array.new }
  # q = (10 ** (logq2/2.0) + 0.5).to_i
  # There is NO conversion to samples here because it is tracing-based!
  for i in 1..$N
    if (i < 380)
      fname = "reports/ak575_" + (logq2 + 97).chr + "-" + i.to_s + "DM.txt"
    else
      fname = "reports/ak575_" + (logq2 + 97).chr + "-" + (i+1).to_s + "DM.txt"
    end
    onepair = getone(fname)
    pair[0].push(onepair[0])
    pair[1].push(onepair[1])
  end
  # FIXME for p=0
  raise "Sample count mismatch" if pair[0].size != pair[1].size
  raise "Wrong sample count" if pair[0].size != $N
  pair
end

# i & 0F  = logq*2  (0..10, 11-15 are unused)
# i & 10  = p   (0=0.2(fn1), 1=0.8(fn2))
data = Array.new(27) { Array.new }
(0..10).each do |i|
  pair = convert(i)
  data[i] = pair[0]
  data[i+0x10] = pair[1]
end

crunched = File.open("data-summary.txt","w")
crunched.puts "logq2 p count mean sd"
dump = File.open("data-denormalized.txt","w")
dump.puts "logq2 p sample"
(0..26).each do |i|
  logq2 = i&0xF
  if logq2 < 11
    p  = i&0x10==0 ? "0" : "1"
    crunched.puts logq2.to_s + " " + p + " " + data[i].size.to_s + " " + mean(data[i]).to_s + " " + sd(data[i]).to_s
    data[i].each {|x| dump.print logq2, " ", p, " ", x, "\n" }
  end
end
