# 2013-08-21 13:48

options(warn=1)

# Requires plotrix package; tested plotrix_3.4-7.tar.gz
library(plotrix)

# logq2 to log10(calls) transformation
callcount <- function(logq2){
  return (1.0+logq2/2.0)
}

data <- read.table("data-summary.txt",header=T)
ylim1 <- min(data$mean)
ylim2 <- max(data$mean)
xlim1 <- callcount(min(data$logq2))
xlim2 <- callcount(max(data$logq2))
xlims <- c(xlim1,xlim2)
ylims <- c(ylim1,ylim2)

# 95% coverage interval for the means
alpha <- 0.05
dof <- data$count[1] - 1
semdiv <- sqrt(data$count[1])
k <- qt(1-alpha/2,dof)
data$meanlo <- data$mean - k * data$sd / semdiv
data$meanhi <- data$mean + k * data$sd / semdiv

makefile <- function(ispdf) {
  if (ispdf) {
    pdf("mean-orig.pdf")
    par(mar=c(4,4,0,0.1))
    cexval <- 0.8
    arrlen <- 0.06
    right_side_labels_horizontal_fudge <- 0.38   # x-axis units (+ = right)
  } else {
    png("mean-orig.png",width=800,height=800,units="px",pointsize=16)
    par(mar=c(4,4,2,0.1))
    cexval <- 1.0
    arrlen <- 0.1
    right_side_labels_horizontal_fudge <- 0.375  # x-axis units (+ = right)
  }

  # ylab=expression(paste(mu,"s"))
  plot (callcount(data$logq2[data$p==0]),data$mean[data$p==0],xlim=xlims,ylim=ylims,pch=16,cex=cexval,col="black",xlab="log10 of number of function calls per program run",ylab="Mean reported self time (ms)")
  points (callcount(data$logq2[data$p==1]),data$mean[data$p==1],pch=15,cex=cexval,col="black")
  
  title ("DroidX Dalvik Debug mean reported self times",line=1)

  legend(1,6e+3,legend=c("fn1 with 2/10 calls",
                         "fn2 with 8/10 calls"),pch=c(16,15))

  # Coverage intervals of the means
  arrows(callcount(data$logq2),data$meanlo,callcount(data$logq2),data$meanhi,angle=90,code=3,length=arrlen,col="black")

  # text(xpd=TRUE,x=xlim2+right_side_labels_horizontal_fudge,y=(gapy2+ylim2-gaplen)/2,srt=270,labels=c("fn2, 8/10 calls"))
  # text(xpd=TRUE,x=xlim2+right_side_labels_horizontal_fudge,y=(gapy1+ylim1)/2,srt=270,labels=c("fn1, 2/10 calls"))
  dev.off()
}

makefile(F)
makefile(T)
