/**
 * Proportionate 
 * Purpose:	Show the effects of self-time fragmentation (the q value) on variability
 * Built on Eclipse for Linux, Ubuntu 12.04 LTS
 * By:	Ramon Luis De Jesus
 * On:	July 10, 2013	2:30pm
 * Note:	Arguments MUST be passed by Proportinate Launcher for this app to run. 
 */
package gov.nist.proportionate;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.WindowManager;
import android.widget.Toast;

public class MainActivity extends Activity {

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		// Keep screen on, even if the device its not charging.
		// Bug fixer for stops in the service.
		getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

		Bundle bundle = null;
		try {
			bundle = getIntent().getExtras();
		} catch (Exception e1) {
			bundle = null;
			e1.printStackTrace();
		}

		String traceName = "";
		long totalLoops = 0L;
		double p = 0.0;
		long d = 0L;
		int bufferSize = 0;

		if (bundle != null) {
			try {
				Log.d("GettingValues", "...");
				if (bundle.getString("traceNameValue") != null
						&& bundle.getString("traceNameValue") != "") {
					traceName = bundle.getString("traceNameValue");
				}

				if (bundle.getDouble("pValue") != 0.0) {
					p = bundle.getDouble("pValue");
				}

				if (bundle.getLong("dValue") != 0L) {
					d = bundle.getLong("dValue");
				}

				if (bundle.getLong("totalLoopsValue") != 0L) {
					totalLoops = bundle.getLong("totalLoopsValue");
				}

				if (bundle.getInt("bufferValue") != 0) {
					bufferSize = bundle.getInt("bufferValue");
				}

			} catch (Exception e) {
				Log.e("ERROR", e.getMessage());
				e.printStackTrace();
			}

			/** Start proportionate Test Case if it passes validation */
			if (p != 0.0 && d != 0L && traceName != "" && totalLoops != 0L
					&& bufferSize != 0) {
				Log.d("Running", "...");

				Log.d("p", Double.toString(p));
				Log.d("d", Long.toString(d));
				Log.d("tracename", traceName);
				Log.d("totalLoops", Long.toString(totalLoops));
				Log.d("bufferSize", Integer.toString(bufferSize));

				proportionate test = new proportionate();
				test.Start(p, d, totalLoops, traceName, bufferSize);

			} else {
				Toast.makeText(this, "Parameters are null or 0.",
						Toast.LENGTH_LONG).show();
				Log.e("ERROR", "Parameters are null or 0.");
				// Parameters are
				Log.e("p", Double.toString(p));
				Log.e("d", Long.toString(d));
				Log.e("tracename", traceName);
				Log.e("totalLoops", Long.toString(totalLoops));
				Log.e("bufferSize", Integer.toString(bufferSize));
			}
		} else {
			Toast.makeText(this,
					"App can only be launched with Proportionate Launcher.",
					Toast.LENGTH_LONG).show();
			Log.e("ERROR",
					"App can only be launched with Proportionate Launcher.");
		}
		finish();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	public void onDestroy() {
		super.onDestroy();
		System.exit(0);
	}

}
