package gov.nist.proportionate;

import android.os.Debug;
import android.os.Environment;

public class proportionate {

	/**
	 * proportionate.java 2013-07-10 3:17PM Reference executable for examining
	 * self-time variability vs. the proportion of self time, enhanced version.
	 * The first one made single invocations of functions that ran for different
	 * amounts of time. This one accomplishes proportionality by instead varying
	 * the number of invocations of functions that run the same length of time
	 * and allows the balance of function length versus iterations to be
	 * adjusted. The first command-line argument is a proportion, 0.0 <= p <=
	 * 1.0. fn1 will get approximately p proportion of self-time. fn2 will get
	 * approximately 1-p. Only multiples of 0.1 are supported currently.
	 * 
	 * The second command-line argument is an integer, 1 <= d <= bignum. This is
	 * the number of cycles to make in the main program, with cycles in the leaf
	 * functions being reduced commensurately.
	 * 
	 * There is an additional factor of 10 conversion resulting from the manner
	 * in which the proportionality is implemented in the main program.
	 * 
	 * A third case not implemented here is where the functions still have
	 * different run times but the balance of function length versus iterations
	 * is adjusted.
	 * 
	 * Old (2013-05-15 3:51PM) Spin was for Windows + Intel Composer XE 2013 +
	 * Microsoft Visual Studio 2012. Language c99.
	 * 
	 * New Spin for Android + DDMS + Eclipse. Language Java.
	 */

	// Configurable TOTALLOOP: how many iterations of the floating-point
	// busy-work will ultimately be done.
	private static long TOTALLOOP = 200000000;

	private long accumulator = 1;
	private double adder = 0;

	/** proportionate start */
	public int Start(double p, long d, long totalLoop, String TraceName,
			int BufferSize) {

		if (totalLoop != 0) {
			TOTALLOOP = totalLoop;
		}

		if (p < 0.0 || p > 1.0 || d == 0L || TraceName == null
				|| TraceName == "") {
			// Say something...
			return 0;
		} else {
			Debug.startMethodTracing(Environment.getExternalStorageDirectory()
					+ "/Traces/" + TraceName, BufferSize);
			long loopcount = (TOTALLOOP / d) / 10;
			double px10 = p * 10;
			double mpx10 = 10 - px10;

			for (long l1 = 0; l1 < d; ++l1) {
				for (double l2 = 0; l2 < px10; ++l2) {
					fn1(loopcount);
				}
				for (double l2 = 0; l2 < mpx10; ++l2) {
					fn2(loopcount);
				}
			}
		}
		// Say something...
		Debug.stopMethodTracing();
		return 0;
	}

	// / Overload of the start function, if programmer just starts the gathering
	// process.
	public int Start() {
		// Modify below parameters for a default overload of the method Start()
		return Start(0.2, 0L, 0L, null, 0);
	}

	/** Functions to call */
	// fn1
	void fn1(long loopcount) {
		for (long looper = 0; looper < loopcount; ++looper) {
			adder += Math.PI * 3;
			accumulator = (long) (accumulator * 3 + adder);
		}
	}

	// fn2
	void fn2(long loopcount) {
		for (long looper = 0; looper < loopcount; ++looper) {
			adder += Math.PI * 3;
			accumulator = (long) (accumulator * 3 + adder);
		}
	}
}
