/**
 * RTD AppLauncher
 * Purpose:	Launch the service and watch in an AsyncTask for any changes in processes.
 * Built on Eclipse for Linux, Ubuntu 12.04 LTS
 * By:	Ramon Luis De Jesus
 * On:	Jun 15, 2013	2:37pm
 * Note:	Uses RootTools-2.6.jar from Stericson (https://code.google.com/p/roottools/)
 */

package gov.nist.proportionatelauncher;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.util.List;

import android.app.ActivityManager;
import android.app.ActivityManager.RunningAppProcessInfo;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.media.RingtoneManager;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Environment;
import android.os.IBinder;
import android.os.StatFs;
import android.os.Vibrator;
import android.support.v4.app.NotificationCompat;
import android.util.Log;
import android.widget.Toast;

public class AppLauncher extends Service {

	/** Global values */
	// To test, package name
	private String packageName = MainActivity.PackageName;

	// To test, uid of package name
	private int uid = 0;

	// To test, pid int
	private int pid = 0;

	// To test, times I have repeated the app
	private int repeated = 0;

	// To test, root log output
	@SuppressWarnings("unused")
	private String output = "";

	// To use time stamps
	MicroTimer microTime;

	// To select d parameters

	// Initiates appcheck AsyncTasks
	AppCheck appCheck = new AppCheck();

	/** Service Methods */
	@Override
	public IBinder onBind(Intent arg0) {
		return null;
	}

	/**
	 * To be run when the btnStart on MainActivity is triggered.
	 */
	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		if (repeated < MainActivity.RepeatTimes) {
			if (repeated == 0) {
				java.util.Date date = new java.util.Date();
				MainActivity.DateStarted = DateFormat.getTimeInstance(
						DateFormat.LONG).format(date);
				Log.d("Proportionate Service", "Started...");
				MessageBox("Proportionate Service Started");
			}

			packageName = MainActivity.PackageName;

			// Start async thread
			appCheck.WantAppCheck = true;
			appCheck.execute(packageName);

			// Service runs until we stop it
			return START_STICKY;
		} else {
			this.stopSelf();
		}
		return START_NOT_STICKY;
	}

	/**
	 * To be run when btnStop on MainActivity is trigerred or when an error
	 * happen and this.stopself() is called.
	 */
	@Override
	public void onDestroy() {

		// Go to sleep for any renaming process remaining.
		try {
			Thread.sleep(MainActivity.Seconds * 2000);
		} catch (InterruptedException e) {
			Log.d("Main", "ERROR SLEEPING");
		}

		// Write array list to a log on sdcard
		// WriteElapsedTimeToSD(MainActivity.ElapsedTimes);

		java.util.Date date = new java.util.Date();
		MainActivity.DateFinished = DateFormat.getTimeInstance(DateFormat.LONG)
				.format(date);
		appCheck.StopAppCheck();
		Log.d("Proportionate Service", "Finished!!!");

		Vibrator v = (Vibrator) getSystemService(Context.VIBRATOR_SERVICE);
		// Vibrate for 500 milliseconds
		v.vibrate(500);

		try {

			RaiseNotification(
					"Proportionate \"" + MainActivity.TraceName
							+ "\" completed!",
					MainActivity.DateStarted + "-" + MainActivity.DateFinished
							+ System.getProperty("line.separator")
							+ "Name of trace: " + "\"" + MainActivity.TraceName
							+ "\"" + System.getProperty("line.separator")
							+ "Package name: " + "\""
							+ MainActivity.PackageName + "\""
							+ System.getProperty("line.separator")
							+ "Amount of traces: " + Integer.toString(repeated)
							+ System.getProperty("line.separator")
							+ "Started on: " + MainActivity.DateStarted
							+ System.getProperty("line.separator")
							+ "Finished at: " + MainActivity.DateFinished);
		} catch (Exception e) {
			Log.d("NOTIF ERROR", e.getMessage());
		}

		repeated = 0;
		MessageBox("Service Stopped");

		if (!EnoughSpace()) {
			MessageBox("Not enough memory on sdcard to proceed with this tests.");
		}

		super.onDestroy();
	}

	/**
	 * Start Asynctask
	 */

	/** AsyncTask to check if app is running */
	public class AppCheck extends AsyncTask<String, Void, Boolean> {

		private Boolean WantAppCheck = true;
		private Boolean TimeAdded = false;

		@Override
		protected Boolean doInBackground(String... params) {
			boolean appRunning = true;

			Log.d("Main", "Sleeping...");
			try {
				Thread.sleep(MainActivity.Seconds * 2500);
			} catch (InterruptedException e) {
				Log.d("Main", "ERROR SLEEPING");
			}
			Log.d("Main", "Resuming...");

			Log.d("App state", "verifying app...");
			while (appRunning && WantAppCheck) {
				ActivityManager activityManager = (ActivityManager) AppLauncher.this
						.getSystemService(ACTIVITY_SERVICE);
				List<RunningAppProcessInfo> procInfos = activityManager
						.getRunningAppProcesses();
				for (int i = 0; i < procInfos.size(); i++) {
					if (procInfos.get(i).processName.equals(params[0])) {
						uid = procInfos.get(i).uid;
						pid = getAppPidByUid(uid);
						String time = (microTime != null)
								&& (microTime.getCurrentTime() != "") ? microTime.Time
								: "null";
						Log.d("App state", "Running. " + time);
						appRunning = true;
						break;
					} else {
						if (!TimeAdded && microTime != null
								&& microTime.getCurrentTime() != ""
								&& runningTest <= MainActivity.dList.size()
								&& lastRunningTest != runningTest
								&& AppLaunched) {
							lastRunningTest = runningTest;
							if (repeated >= 0) {
								AppendToFile(
										MainActivity.TraceName
												+ "_"
												+ MainActivity.dList.get(
														runningTest - 1)
														.getKey() + "-",
										new String(microTime.Time));
							}

							AppLaunched = false;
						}
						appRunning = false;
					}
				}
				if (!appRunning) {
					Log.d("App state", "Not running.");
				}
			}
			return appRunning;
		}

		public void StopAppCheck() {
			WantAppCheck = false;
		}

		public void SetTimeAdded(Boolean set) {
			TimeAdded = set;
		}

		@Override
		protected void onPostExecute(Boolean isRunning) {
			super.onPostExecute(isRunning);

			if (!isRunning) {
				if (WantAppCheck) {
					LaunchApp(packageName);
				}
			}
		}
	}

	/**
	 * End AsyncTask
	 */

	/**
	 * App management
	 */

	Integer runningTest = 0;
	Integer lastRunningTest = 5;
	Boolean AppLaunched = false;

	// Launch a user custom selected app.
	void LaunchApp(String AppPackageNameToStart) {
		if (repeated < MainActivity.RepeatTimes && EnoughSpace()) {
			Log.d("App state", "Launching: " + AppPackageNameToStart);
			try {
				Intent i = new Intent();
				PackageManager manager = getPackageManager();
				i = manager.getLaunchIntentForPackage(AppPackageNameToStart);
				i.addCategory(Intent.CATEGORY_LAUNCHER);

				if (runningTest < MainActivity.dList.size()) {
					/**
					 * Pass arguments
					 */
					i.putExtra("traceNameValue", MainActivity.TraceName + "_"
							+ MainActivity.dList.get(runningTest).getKey()
							+ "-" + (Integer.toString(repeated + 1)));

					Log.e("traceName", MainActivity.TraceName + "_"
							+ MainActivity.dList.get(runningTest).getKey()
							+ "-" + (Integer.toString(repeated + 1)));
					i.putExtra("pValue", MainActivity.p);
					Log.d("pValue", Double.toString(MainActivity.p));
					i.putExtra("dValue", MainActivity.dList.get(runningTest)
							.getValue());
					Log.d("dValue", Long.toString(MainActivity.dList.get(
							runningTest).getValue()));
					i.putExtra("totalLoopsValue", MainActivity.TotalLoops);
					Log.d("totalLoopsValue",
							Long.toString(MainActivity.TotalLoops));
					i.putExtra("bufferValue", MainActivity.BufferSize);
					Log.d("bufferValue",
							Integer.toString(MainActivity.BufferSize));

					appCheck.SetTimeAdded(false);
					microTime = new MicroTimer();

					runningTest++;
					Log.e("Test number:", Integer.toString(runningTest)
							+ " in progress...");

					startActivity(i);
					AppLaunched = true;

				} else {
					runningTest = 0;
					repeated++;
					Log.d("Repeat", Integer.toString(repeated));
				}
			} catch (Exception ex) {
				this.stopSelf();
				MessageBox("Error launching: " + AppPackageNameToStart + "/n"
						+ ex.getLocalizedMessage() + "/n" + ex.getMessage());
			}
			if (repeated != 0 || runningTest != 0) {
				try {
					appCheck = new AppCheck();
					appCheck.execute(packageName);
				} catch (Exception e) {
					Log.e(e.getLocalizedMessage(), e.getMessage());
				}
			} else {
				this.stopSelf();
			}
		} else {
			this.stopSelf();
		}
	}

	// Gets AppId with uid.
	// AppId good for kill or launch of an app.
	int getAppPidByUid(int uid) {
		ActivityManager manager = (ActivityManager) this
				.getSystemService(Context.ACTIVITY_SERVICE);
		for (RunningAppProcessInfo processInfo : manager
				.getRunningAppProcesses()) {
			if (processInfo.uid == this.uid) {
				return processInfo.pid;
			}
		}
		return 0;
	}

	/**
	 * File writing, logs, etc
	 */

	// Write lines to the text on sdcard
	void AppendToFile(String NameOfFile, String Message) {
		File serviceTimesFile = new File(Environment
				.getExternalStorageDirectory().getPath()
				+ "/Traces/"
				+ NameOfFile + "-ServiceTimes.csv");
		if (!serviceTimesFile.exists()) {
			try {
				serviceTimesFile.createNewFile();
				try {
					// BufferedWriter for performance, true to set append to
					// file flag
					BufferedWriter buf = new BufferedWriter(new FileWriter(
							serviceTimesFile, true));
					buf.append("ServiceTimes");
					buf.newLine();
					buf.append(Message);
					buf.newLine();
					buf.flush();
					buf.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else {
			try {
				// BufferedWriter for performance, true to set append to file
				// flag
				BufferedWriter buf = new BufferedWriter(new FileWriter(
						serviceTimesFile, true));
				buf.append(Message);
				buf.newLine();
				buf.flush();
				buf.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	Boolean EnoughSpace() {
		StatFs stat = new StatFs(Environment.getExternalStorageDirectory()
				.getPath());
		long bytesAvailable = (long) stat.getBlockSize()
				* (long) stat.getBlockCount();
		// long megAvailable = bytesAvailable / 1048576;

		if (bytesAvailable <= (MainActivity.BufferSize * 2)) {
			appCheck.StopAppCheck();

			Log.e("Error",
					"Not enough memory on sdcard to proceed with this tests.");
			try {
				Thread.sleep(MainActivity.Seconds * 2000);
			} catch (InterruptedException e) {
				Log.d("Main", "ERROR SLEEPING");
			}
			if (MainActivity.DateFinished != "") {
				this.stopSelf();
			}
			return false;
		} else {
			return true;
		}
	}

	/**
	 * Misc
	 * 
	 */
	public void MessageBox(String message) {
		Toast.makeText(this, message, Toast.LENGTH_SHORT).show();
	}

	public void RaiseNotification(String titleNotification,
			String mesgNotification) {
		Intent intent = new Intent(this, NotificationReceiverActivity.class);
		@SuppressWarnings("unused")
		PendingIntent pIntent = PendingIntent.getActivity(this, 0, intent, 0);

		if (Build.VERSION.SDK_INT >= 11) {

			// Build notification
			// Actions are just fake
			Notification noti = new Notification.Builder(this)
					.setContentTitle(titleNotification)
					.setContentText(mesgNotification)
					.setSound(
							RingtoneManager
									.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION))
					.setStyle(
							new Notification.BigTextStyle()
									.bigText(mesgNotification))
					.setSmallIcon(R.drawable.ic_launcher).build();

			NotificationManager notificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);

			// Hide the notification after its selected
			noti.flags |= Notification.FLAG_AUTO_CANCEL;

			notificationManager.notify(0, noti);
		} else {
			NotificationCompat.Builder builder = new NotificationCompat.Builder(
					this)
					.setAutoCancel(true)
					.setContentTitle(titleNotification)
					.setContentText(mesgNotification)
					.setSound(
							RingtoneManager
									.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION))
					.setSmallIcon(R.drawable.ic_launcher);

			PendingIntent in = PendingIntent.getActivity(
					getApplicationContext(), 0, intent, 0);
			builder.setContentIntent(in);

			NotificationManager notificationManager = null;
			try {
				notificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
			} catch (Exception e) {
				Log.e("ERROR", e.getMessage());
			}

			try {
				notificationManager.notify(0, builder.build());
			} catch (Exception e) {
				Log.e("ERROR", e.getMessage());
			}
		}
	}
}
