# 2013-08-21 14:07
# bootstrap-run.R runs MCM and produces bootstrap-data-summary.txt.
# This script reads that file and produces the plots.

options(warn=1)

# Requires plotrix package; tested plotrix_3.4-7.tar.gz
library(plotrix)

stats <- read.table("bootstrap-data-summary.txt",header=T)

# logq2 to log10(cpu-cycles) transformation
cyclecount <- function(logq2){
  logq2/2.0
}

ylim1 <- 0
ylim2 <- 12
xlim1 <- cyclecount(min(stats$logq2))
xlim2 <- cyclecount(max(stats$logq2))
xlims <- c(xlim2,xlim1)
ylims <- c(ylim1,ylim2)

makemeanfile <- function(ispdf) {
  if (ispdf) {
    pdf("mean-bootstrap.pdf")
    par(mar=c(4,4,0.1,0.1))
    arrlen <- 0.06
    right_side_labels_horizontal_fudge <- 0.38   # x-axis units (+ = right)
  } else {
    png("mean-bootstrap.png",width=800,height=800,units="px",pointsize=16)
    par(mar=c(4,4,2,0.1))
    arrlen <- 0.1
    right_side_labels_horizontal_fudge <- 0.375  # x-axis units (+ = right)
  }

  plot (cyclecount(stats$logq2[stats$p==0]),log(stats$mean[stats$p==0]),xlim=xlims,ylim=ylims,pch=16,col="black",xlab="log10 of sampling period (cpu-cycles)",ylab="log10 of mean sample count",xaxt="n")
  points (cyclecount(stats$logq2[stats$p==1]),log(stats$mean[stats$p==1]),pch=15,col="black")
  title ("Effect of sampling frequency on mean sample count for perf",line=1)

  axis(1,at=cyclecount(stats$logq2[stats$p==0]),labels=cyclecount(stats$logq2[stats$p==0]))
  legend(8.5,11,legend=c("fn1 with 2/10 calls",
                         "fn2 with 8/10 calls"),pch=c(16,15))

  # Coverage intervals of the means
  arrows(cyclecount(stats$logq2),log(stats$meanlo),cyclecount(stats$logq2),log(stats$meanhi),angle=90,code=3,length=arrlen,col="black")
  dev.off()
}

makesdfile <- function(ispdf,isrel) {
  if (isrel) {
    ylab <- "Sample count standand deviation / mean"
    legx <- 6.5
    legy <- 0.2
    pdfnam <- "sdrel-bootstrap.pdf"
    pngnam <- "sdrel-bootstrap.png"
  } else {
    ylab <- "Standard deviation of sample counts"
    legx <- 8
    legy <- 10
    pdfnam <- "sd-bootstrap.pdf"
    pngnam <- "sd-bootstrap.png"
  }

  if (ispdf) {
    pdf(pdfnam)
    par(mar=c(4,4,0.1,0.1))
    arrlen <- 0.06
    plot(cyclecount(stats$logq2[stats$p==0]),stats$sd[stats$p==0],xlab="log10 of sampling period (cpu-cycles)",ylab=ylab,pch=16,col="black",xlim=xlims,ylim=c(0,max(stats$sd,stats$sdhi)),xaxt="n")
  } else {
    png(pngnam,width=800,height=800,units="px",pointsize=16)
    par(mar=c(4,4,2,0.1))
    arrlen <- 0.1
    plot(cyclecount(stats$logq2[stats$p==0]),stats$sd[stats$p==0],xlab="log10 of sampling period (cpu-cycles)",ylab=ylab,pch=16,col="black",main="Effect of sampling frequency on sample count standard deviation for perf",xlim=xlims,ylim=c(0,max(stats$sd,stats$sdhi)),xaxt="n")
  }

  # Actual sds
  points(cyclecount(stats$logq2[stats$p==1]),stats$sd[stats$p==1],pch=15,col="black")

  # Coverage intervals of the sds
  arrows(cyclecount(stats$logq2[stats$p==0]),stats$sdlo[stats$p==0],cyclecount(stats$logq2[stats$p==0]),stats$sdhi[stats$p==0],angle=90,code=3,length=arrlen,col="black")
  arrows(cyclecount(stats$logq2[stats$p==1]),stats$sdlo[stats$p==1],cyclecount(stats$logq2[stats$p==1]),stats$sdhi[stats$p==1],angle=90,code=3,length=arrlen,col="black")

  axis(1,at=cyclecount(stats$logq2[stats$p==0]),labels=cyclecount(stats$logq2[stats$p==0]))
  legend(legx,legy,legend=c("fn1 with 2/10 calls",
                            "fn2 with 8/10 calls"),pch=c(16,15))
  dev.off()
}

makesdfile(F,F)
makesdfile(T,F)
makemeanfile(F)
makemeanfile(T)

stats$sd <- stats$sd / stats$mean
stats$sdlo <- stats$sdlo / stats$mean
stats$sdhi <- stats$sdhi / stats$mean
makesdfile(F,T)
makesdfile(T,T)
