# 2013-08-21 13:59
# Rscript makeplots.R

# Requires plotrix package; tested plotrix_3.4-7.tar.gz
library(plotrix)

x2 <- read.table("crunched-2.txt",header=T)
x8 <- read.table("crunched-8.txt",header=T)

# 95% coverage interval for the standard deviations
alpha <- 0.05
if (x2$count[1] != x8$count[1]) stop ("count mismatch")
dof <- x2$count[1] - 1
x2$sdlo <- x2$sd * sqrt(dof/qchisq(1-alpha/2,dof))
x2$sdhi <- x2$sd * sqrt(dof/qchisq(alpha/2,dof))
x8$sdlo <- x8$sd * sqrt(dof/qchisq(1-alpha/2,dof))
x8$sdhi <- x8$sd * sqrt(dof/qchisq(alpha/2,dof))

# 95% coverage interval for the means
semdiv <- sqrt(x2$count[1])
k <- qt(1-alpha/2,dof)
x2$meanlo <- x2$mean - k * x2$sd / semdiv
x2$meanhi <- x2$mean + k * x2$sd / semdiv
x8$meanlo <- x8$mean - k * x8$sd / semdiv
x8$meanhi <- x8$mean + k * x8$sd / semdiv

# Circles and squares again.
fn1pch <- 16
fn2pch <- 15

makemeansfile <- function(ispdf) {
  if (ispdf) {
    pdf("sampfreqadjmeans.pdf")
    par(mar=c(4,4,0.1,0.1))
    arrlen <- 0.06
    plot(x8$log.q.,log(x8$mean),xlab="log10 of sampling period (cpu-cycles)",ylab="log10 of mean sample count",pch=fn2pch,xlim=c(max(x8$log.q.),min(x8$log.q.)),ylim=c(0,12),xaxt="n")
  } else {
    png("sampfreqadjmeans.png",width=800,height=800,units="px",pointsize=16)
    par(mar=c(4,4,2,0.1))
    arrlen <- 0.1
    plot(x8$log.q.,log(x8$mean),xlab="log10 of sampling period (cpu-cycles)",ylab="log10 of mean sample count",pch=fn2pch,xlim=c(max(x8$log.q.),min(x8$log.q.)),ylim=c(0,12),main="Effect of sampling frequency on mean sample count for perf",xaxt="n")
  }

  points(x2$log.q.,log(x2$mean),pch=fn1pch)

  # Coverage intervals of the means
  arrows(x2$log.q.,log(x2$meanlo),x2$log.q.,log(x2$meanhi),angle=90,code=3,length=arrlen,col="black")
  arrows(x8$log.q.,log(x8$meanlo),x8$log.q.,log(x8$meanhi),angle=90,code=3,length=arrlen,col="black")

  axis(1,at=x2$log.q.,labels=x2$log.q.)
  legend(8.5,11,legend=c("fn1 with 2/10 calls","fn2 with 8/10 calls"),pch=c(fn1pch,fn2pch))
  dev.off()
}

makemeansfile(T)
makemeansfile(F)

makesdsfile <- function(ispdf) {
  if (ispdf) {
    pdf("sampfreqadjsds.pdf")
    par(mar=c(4,4,0.1,0.1))
    arrlen <- 0.06
    plot(x8$log.q.,x8$sd,xlab="log10 of sampling period (cpu-cycles)",ylab="Standard deviation of sample counts",pch=fn2pch,xlim=c(max(x8$log.q.),min(x8$log.q.)),ylim=c(0,max(x8$sdhi)),xaxt="n")
  } else {
    png("sampfreqadjsds.png",width=800,height=800,units="px",pointsize=16)
    par(mar=c(4,4,2,0.1))
    arrlen <- 0.1
    plot(x8$log.q.,x8$sd,xlab="log10 of sampling period (cpu-cycles)",ylab="Standard deviation of sample counts",pch=fn2pch,xlim=c(max(x8$log.q.),min(x8$log.q.)),ylim=c(0,max(x8$sdhi)),main="Effect of sampling frequency on sample count standard deviation for perf",xaxt="n")
  }

  points(x2$log.q.,x2$sd,pch=fn1pch)

  # Coverage intervals of the sds
  arrows(x2$log.q.,x2$sdlo,x2$log.q.,x2$sdhi,angle=90,code=3,length=arrlen,col="black")
  arrows(x8$log.q.,x8$sdlo,x8$log.q.,x8$sdhi,angle=90,code=3,length=arrlen,col="black")

  axis(1,at=x2$log.q.,labels=x2$log.q.)
  legend(8,10,legend=c("fn1 with 2/10 calls","fn2 with 8/10 calls"),pch=c(fn1pch,fn2pch))
  dev.off()
}

makesdsfile(T)
makesdsfile(F)

x2$sd   <- x2$sd   / x2$mean
x2$sdhi <- x2$sdhi / x2$mean
x2$sdlo <- x2$sdlo / x2$mean
x8$sd   <- x8$sd   / x8$mean
x8$sdhi <- x8$sdhi / x8$mean
x8$sdlo <- x8$sdlo / x8$mean

makesdsrelfile <- function(ispdf) {
  if (ispdf) {
    pdf("sampfreqadjsdsrel.pdf")
    par(mar=c(4,4,0.1,0.1))
    arrlen <- 0.06
    plot(x8$log.q.,x8$sd,xlab="log10 of sampling period (cpu-cycles)",ylab="Sample count standand deviation / mean",pch=fn2pch,xlim=c(max(x8$log.q.),min(x8$log.q.)),ylim=c(0,max(x8$sdhi,x2$sdhi)),xaxt="n")
  } else {
    png("sampfreqadjsdsrel.png",width=800,height=800,units="px",pointsize=16)
    par(mar=c(4,4,2,0.1))
    arrlen <- 0.1
    plot(x8$log.q.,x8$sd,xlab="log10 of sampling period (cpu-cycles)",ylab="Sample count standand deviation / mean",pch=fn2pch,xlim=c(max(x8$log.q.),min(x8$log.q.)),ylim=c(0,max(x8$sdhi,x2$sdhi)),main="Effect of sampling frequency on sample count standard deviation for perf",xaxt="n")
  }

  points(x2$log.q.,x2$sd,pch=fn1pch)

  # Coverage intervals of the sds
  arrows(x2$log.q.,x2$sdlo,x2$log.q.,x2$sdhi,angle=90,code=3,length=arrlen,col="black")
  arrows(x8$log.q.,x8$sdlo,x8$log.q.,x8$sdhi,angle=90,code=3,length=arrlen,col="black")

  axis(1,at=x2$log.q.,labels=x2$log.q.)
  legend(6.5,0.2,legend=c("fn1 with 2/10 calls","fn2 with 8/10 calls"),pch=c(fn1pch,fn2pch))
  dev.off()
}

makesdsrelfile(T)
makesdsrelfile(F)
