#!/usr/bin/ruby
# 2013-07-08 11:49

# Number of samples for each data set
$N = 205

# If true, stop at $N or end of file, whichever comes first.
# If false, read to end of file.
# Either way, complain if don't get $N samples for everything.
$TRUNCATE = false

# Number of functions that are being tracked.
$NUMGLOM = 2

# This does not do the corrected sample standard deviation (n-1).
# descriptive_statistics-1.1.1.gem from https://github.com/thirtysixthspan/descriptive_statistics 2013-05-14 13:58
# gem install descriptive_statistics-1.1.1.gem
# require 'descriptive_statistics'

def sum(x)
  x.inject(:+).to_f
end

def mean(x)
  sum(x)/x.size
end

def sd(x)
  xm = mean(x)
  v = x.map{ |sample| (xm - sample) ** 2 }.inject(:+) / (x.size - 1.0)
  Math.sqrt(v)
end

def fncount2bucketi(fncount)
  fncount / $NUMGLOM
end

# convert returns a pair (actually an Array of size 2) of Arrays.  The first
# one contains the sample counts from fn1; the second from fn2.
def convert(q,load)
  pair = Array.new(2) { Array.new }
  fname = q.to_s + "_" + load + ".txt"
  puts "Reading " + fname
  File.open(fname,"r") do |file|
    fncount = 0
    file.lines do |line|
      line.chomp!
      if line[0..1] == "fn"
        i = line[2].to_i - 1
        raise "AXE format error" if i < 0 or i > 1
        ms = (line.split(' ')[2].to_f * 1000 + 0.5).to_i
        # Approximate conversion to sample count
        samples = 1.3 * ms
        pair[i].push(samples)
        fncount += 1
        break if $TRUNCATE and fncount2bucketi(fncount) == $N
      end
    end
  end
  # FIXME for p=0
  raise "Sample count mismatch" if pair[0].size != pair[1].size
  raise "Wrong sample count" if pair[0].size != $N
  pair
end

# bit 0   p  (0=0.2(fn1), 1=0.8(fn2))
# bit 1   q  (0=1, 1=100000)
# bit 2   smp+load (0=no, 1=yes)
data = Array.new(8) { Array.new }
(0..3).each do |i|
  q = (i&0x1==0 ? "1" : "100000")
  load = (i&0x2==0 ? "smp_noload" : "nosmp_load")
  pair = convert(q,load)
  data[2*i]   = pair[0]
  data[2*i+1] = pair[1]
end

crunched = File.open("data-summary.txt","w")
crunched.puts "q p load count mean sd"
dump = File.open("data-denormalized.txt","w")
dump.puts "q p load sample"
(0..7).each do |i|
  p = (i&0x1==0 ? "0" : "1")
  q = (i&0x2==0 ? "0" : "1")
  load = (i&0x4==0 ? "0" : "1")
  crunched.puts q + " " + p + " " + load + " " + data[i].size.to_s + " " + mean(data[i]).to_s + " " + sd(data[i]).to_s
  data[i].each {|x| dump.print q, " ", p, " ", load, " ", x, "\n" }
end
