# 2013-08-21 14:48
# This script runs MCM and produces bootstrap-data-summary.txt.
# bootstrap-plot.R reads that file and produces the plots.

# Configurable:  Number of Monte Carlo trials.
M <- 1000000

# Configurable:  Alpha (1 - coverage probability).
alpha <- 1 - 0.95^(1/126^2)

N <- 100

data <- read.table("data-denormalized.txt",header=T)
stats <- aggregate(data, list(dir=data$dir,opt=data$opt,fn=data$fn), FUN=mean)[,4:7]
colnames(stats)[4] <- "mean";
temp <- aggregate(data, list(dir=data$dir,opt=data$opt,fn=data$fn), FUN=sd)[,4:7]
stats$sd <- temp$sample

outputplot <- function(parm,filename,data,interval) {
  # bw.nrd0 is the default estimator of density().
  bw <- bw.nrd0(data)
  png(filename,width=800,height=800,units="px",pointsize=16)
  par(mar=c(4,4,1,0.5))
  plot(density(data,bw=bw),lwd=2.5,xlab=parm,main=sprintf("Bandwidth = %0.4f", bw))
  abline(v=interval,col="blue")
  dev.off()
}

# Return:  estmean meanlo meanhi estsd sdlo sdhi
dogroup <- function(dir,opt,fn) {
  gdata <- data[data$dir==dir & data$opt==opt & data$fn==fn, 'sample']
  if (length(gdata) != N) stop ("Unexpected count")

  cat(sprintf("Running %d Monte Carlo trials for dir=%d opt=%d fn=%d (N=%d)...\n", M, dir, opt, fn, N))
  means <- rep(NA, M)
  sds   <- rep(NA, M)
  for (i in 1:M) {
    sampvec  <- sample(gdata,N,replace=T)
    means[i] <- mean(sampvec)
    sds[i]   <- sd(sampvec)
  }
  quantiles <- c(alpha/2, 1-alpha/2)
  meanqs    <- quantile(means, quantiles)
  sdqs      <- quantile(sds, quantiles)

  # Save pictures of the output distributions with the coverage intervals.
  outputplot("sample count mean",sprintf("outputdist-mean-dir%d-opt%d-fn%d.png", dir, opt, fn),means,meanqs)
  outputplot("sample count SD",sprintf("outputdist-SD-dir%d-opt%d-fn%d.png", dir, opt, fn),sds,sdqs)

  c(mean(means),meanqs,mean(sds),sdqs)
}

for (dir in min(data$dir):max(data$dir)) {
  for (opt in min(data$opt):max(data$opt)) {
    for (fn in min(data$fn):max(data$fn)) {
      MCMout <- dogroup(dir,opt,fn)
      stats$estmean[stats$dir==dir & stats$opt==opt & stats$fn==fn] <- MCMout[1]
      stats$meanlo[stats$dir==dir & stats$opt==opt & stats$fn==fn] <- MCMout[2]
      stats$meanhi[stats$dir==dir & stats$opt==opt & stats$fn==fn] <- MCMout[3]
      stats$estsd[stats$dir==dir & stats$opt==opt & stats$fn==fn] <- MCMout[4]
      stats$sdlo[stats$dir==dir & stats$opt==opt & stats$fn==fn] <- MCMout[5]
      stats$sdhi[stats$dir==dir & stats$opt==opt & stats$fn==fn] <- MCMout[6]
    }
  }
}
statsout <- file("bootstrap-data-summary.txt",open="wt")
write.table(stats,file=statsout,quote=F,row.names=F)
close(statsout)
