# 2013-08-21 09:51

# Requires plotrix package; tested plotrix_3.4-7.tar.gz
library(plotrix)

data <- read.table("data-denormalized.txt",header=T)

optnames <- c("1-gprof","2-perf","3-perf-PEBS","4-operf","5-opcontrol","6-perf")
N <- 100
k <- qt((1+0.95^(1/126^2))/2,N-1)

dosubplot <- function(dir,opt) {
  t <- data[data$dir==dir & data$opt==opt,c('fn','sample')]
  if (nrow(t) != 700) stop ("Unexpected count")

  stats <- aggregate(t, list(fn=t$fn), FUN=mean)[,c(1,3)]
  colnames(stats)[2] <- "mean";
  temp <- aggregate(t, list(fn=t$fn), FUN=sd)[,c(1,3)]
  stats$sd <- temp$sample

  stats$um <- stats$sd / sqrt(N) * k
  stats$meanlo <- stats$mean - stats$um
  stats$meanhi <- stats$mean + stats$um

  op <- par(mar=c(0,1,1,0),mgp=c(0,0,0))
  ymax=max(stats$meanhi)
  plot(stats$mean,xaxt="n",xlab="",ylab="",type="p",pch=20,ylim=c(0,ymax),main=optnames[opt])
  arrlen <- 0.04
  arrows(stats$fn,stats$meanlo,stats$fn,stats$meanhi,angle=90,code=3,length=arrlen,col="black")
}

pdf('means.pdf',height=6.5,width=9)
par(mfrow=c(3,6))
for (dir in min(data$dir):max(data$dir)) {
  for (opt in min(data$opt):max(data$opt)) {
    dosubplot(dir,opt)
  }
}
dev.off()

png("means.png",width=800,height=800,units="px",pointsize=16)
par(mfrow=c(3,6))
for (dir in min(data$dir):max(data$dir)) {
  for (opt in min(data$opt):max(data$opt)) {
    dosubplot(dir,opt)
  }
}
dev.off()
