/* demo.c  2013-03-11 09:47
   Reference executable for demonstrating uniform random distribution of
   samples within a function to validate the Poisson uncertainty argument.

   Optimization does not eliminate asm("nop").  However, -O0 includes
   unwanted frame pointer code.

   Uses Boost preprocessor library and _Static_assert.
   Original versions tested:  gcc 4.7.2, Boost 1.51.0.
*/

// Configurable INNERLOOP:  how many repetitions of boring_instruction in
// function boring_function.
#ifndef INNERLOOP
#define INNERLOOP 256
#endif

// Configurable OUTERLOOP:  how many invocations of boring_function by main.
#ifndef OUTERLOOP
#define OUTERLOOP 30000000000ULL
#endif

#include <stdio.h>
#include <boost/preprocessor/repetition/repeat.hpp>
#include <boost/preprocessor/config/limits.hpp>
_Static_assert(INNERLOOP <= BOOST_PP_LIMIT_REPEAT, "INNERLOOP too big");

// The argument deals with the time dimension, but our observable is in the
// space dimension (the instruction pointer).  To maximize the comparability
// of these two, we generate a function containing many boring repetitions of
// a boring instruction that should have fairly consistent execution time.
#define boring_instruction(unused1,unused2,unused3) asm("nop");
void boring_function() {
  BOOST_PP_REPEAT (INNERLOOP, boring_instruction, "unused")
}

int main() {
  unsigned long long looper;
  FILE *fp;
  int r;

  fp = fopen ("/dev/urandom", "r");
  if (fp == NULL) {
    perror ("/dev/urandom");
    return -1;
  }

  for (looper=0; looper<OUTERLOOP; ++looper) {
    boring_function();
    r = fgetc(fp);
    if (r == EOF) {
      fprintf (stderr, "Error reading from /dev/urandom\n");
      return -1;
    }
    r /= 16;
    while (r > 0) {
      asm("nop");
      --r;
    }
  }
  return 0;
}
